# coding:utf-8
from spyne.decorator import rpc

from . import proxy
from . import entities


@rpc(
    entities.NotBlankUnicode,
    entities.NotBlankUnicode,
    entities.Predicate,
    entities.BlankUnicode,
    _returns=entities.getSlotsEntityResponse,
    _out_message_name='getSlotsResponse',
)
def getSlotsRequest(self, organizationId, userType, filter, attributes):
    u"""
    Функция обработки запроса getSlotsRequest
    Метод getSlots - Запросить свободные слоты времени.
    """

    request = entities.getSlotsEntityRequest(
        organizationId=organizationId,
        userType=userType,
        filter=filter,
        attributes=attributes
    )

    return proxy.GetSlotsProxy(
        self, entities.getSlotsEntityResponse, request
    ).process()


@rpc(
    entities.BaseEntityRequest,
    _returns=entities.getRefsListEntityResponse,
    _out_message_name='getRefsListResponse',
)
def getRefsListRequest(self, BaseEntityRequest):
    u"""
    Функция обработки запроса getRefsListRequest
    Метод getRefsList - Метод позволяет ЕПГУ получать информацию о списке
    справочников, имеющихся в системе. Используется для первоначального
    наполнения базы справочной информации ЕПГУ и отображения на этапе
    конфигурирования услуги.
    """
    return proxy.GetRefsListProxy(
        self, entities.getRefsListEntityResponse, BaseEntityRequest
    ).process()


@rpc(
    entities.NotBlankUnicode.customize(max_occurs='unbounded', min_occurs=0),
    _returns=entities.getRefAttributesListEntityResponse,
    _out_message_name='getRefAttributesListResponse',
)
def getRefAttributesListRequest(self, refNames):
    u"""
    Функция обработки запроса getRefAttributesListRequest
    Метод GetRefAttributesList позволяет получать информацию об атрибутах
    конкретного справочника. Используется для первоначального наполнения базы
    справочной информации ЕПГУ и отображения на этапе конфигурирования услуги.
    В рамках услуги "Запись на прием к специалисту" требуется получение
    атрибутов справочника "Учреждение" и "Причина записи на прием" и
    "Тип документа, удостоверяющего личность".
    """

    request = entities.getRefAttributesListEntityRequest(
        refNames=refNames
    )

    return proxy.GetRefAttributesListProxy(
        self, entities.getRefAttributesListEntityResponse, request
    ).process()


@rpc(
    entities.NotBlankUnicode,
    entities.BlankUnicode,
    entities.NotBlankUnicode,
    entities.BlankUnicode.customize(max_occurs='unbounded', min_occurs=0),
    entities.Predicate,
    entities.BlankUnicode.customize(max_occurs='unbounded', min_occurs=0),
    entities.NotBlankUnicode,
    entities.NotBlankUnicode,
    entities.BlankUnicode,
    _returns=entities.getRefItemsListEntityResponse,
    _out_message_name='getRefItemsListResponse',
)
def getRefItemsListRequest(
        self, refName, parentRefItemValue, treeFiltering, selectAttributes,
        filter, orderBy, pageNum, pageSize, version,
):
    u"""
    Функция обработки запроса getRefItemsListRequest
    Метод позволяет получать информацию о данных справочника.
    Используется для первоначального наполнения базы справочной
    информации ЕПГУ, а также для последующей синхронизации.
    """

    request = entities.getRefItemsListEntityRequest(
        refName=refName,
        parentRefItemValue=parentRefItemValue,
        treeFiltering=treeFiltering,
        selectAttributes=selectAttributes,
        filter=filter,
        orderBy=orderBy,
        pageNum=pageNum,
        pageSize=pageSize,
        version=version
    )
    return proxy.GetRefItemsListProxy(
        self, entities.getRefItemsListEntityResponse, request
    ).process()


@rpc(
    entities.NotBlankUnicode,
    entities.BlankUnicode,
    entities.SlotData,
    entities.NotBlankUnicode,
    _returns=entities.UpdateBookingDetailsResponse,
    _out_message_name='UpdateBookingDetailsResponse',
)
def updateBookingDetailsRequest(self, bookId, esiaId, timeSlot, status):
    u"""
    Переносит прием на другой слот.
    """

    request = entities.UpdateBookingDetailsRequest(
        bookId=bookId,
        esiaId=esiaId,
        timeSlot=timeSlot,
        status=status
    )

    return proxy.UpdateBookingDetailsProxy(
        self, entities.UpdateBookingDetailsResponse, request
    ).process()


@rpc(
    entities.NotBlankUnicode,
    entities.BlankUnicode,
    _returns=entities.CancelBookingResponse,
    _out_message_name='CancelBookingResponse'
)
def cancelBookingRequest(self, bookId, esiaId):
    u"""
    Обработка запроса на отмену бронирования записи к специалисту.
    """
    request = entities.CancelBookingRequest(
        bookId=bookId,
        esiaId=esiaId,
    )

    return proxy.CancelBookingProxy(
        self, entities.CancelBookingResponse, request
    ).process()


@rpc(
    entities.NotBlankUnicode,
    entities.NotBlankUnicode,
    entities.NotBlankUnicode.customize(max_occurs='unbounded', min_occurs=0),
    entities.Enum(*entities.USER_TYPES, type_name='userType'),
    entities.BlankUnicode,
    entities.BlankUnicode,
    entities.Array(entities.NotBlankUnicode),
    entities.BlankUnicode,
    entities.BlankUnicode,
    entities.BlankUnicode,
    entities.BlankUnicode,
    entities.BlankUnicode,
    entities.BlankUnicode,
    entities.BlankUnicode,
    entities.DocumentType,
    entities.BlankUnicode,
    entities.BlankUnicode,
    entities.BlankUnicode,
    entities.BlankUnicode,
    entities.Date,
    entities.DocumentType,
    entities.BlankUnicode,
    entities.NotBlankUnicode.customize(max_occurs='unbounded', min_occurs=0),
    entities.Boolean,
    entities.Integer,
    entities.Array(entities.Attribute),
    _returns=entities.BookResponse,
    _out_message_name='BookResponse',
)
def bookRequest(
        self, bookId, slotId, organizationId, userType, declarantType, esiaId,
        serviceId, lastName, firstName, middleName, email, mobilePhone,
        userSnils, userLogin, userDoc, caseNumber, ChildSurname, ChildName,
        ChildMiddlename, ChildBirthDate, ChildDoc, reasonId, areaId, 
        preliminaryReservation, preliminaryReservationPeriod, attributes
        
):
    u"""
    Обработка запрос на бронирование на запись к специалисту.
    """

    request = entities.BookRequest(
        bookId=bookId,
        slotId=slotId,
        organizationId=organizationId,
        userType=userType,
        declarantType=declarantType,
        esiaId=esiaId,
        serviceId=serviceId,
        lastName=lastName,
        firstName=firstName,
        middleName=middleName,
        email=email,
        mobilePhone=mobilePhone,
        userSnils=userSnils,
        userLogin=userLogin,
        userDoc=userDoc,
        caseNumber=caseNumber,
        ChildSurname=ChildSurname,
        ChildName=ChildName,
        ChildMiddlename=ChildMiddlename,
        ChildBirthDate=ChildBirthDate,
        ChildDoc=ChildDoc,
        reasonId=reasonId,
        areaId=areaId,
        preliminaryReservation=preliminaryReservation,
        preliminaryReservationPeriod=preliminaryReservationPeriod,
        attributes=attributes,
    )
    
    return proxy.BookProxy(self, entities.BookResponse, request).process()


@rpc(
    entities.NotBlankUnicode,
    entities.NotBlankUnicode,
    entities.NotBlankUnicode,
    _returns=entities.GetBookingStatusResponse,
    _out_message_name='GetBookingResponse',
)
def getBookingStatusRequest(self, bookId, esiaId, status):
    u"""
    Метод book начинает отправляется в систему в момент выбора необходимого
    слота времени и его бронирования на портале гос.услуг
    """

    request = entities.GetBookingStatusRequest(
        bookId=bookId,
        esiaId=esiaId,
        status=status,
    )

    return proxy.GetBookingStatusProxy(
        self, entities.GetBookingStatusResponse, request
    ).process()
